-- Use Chandra aspect solutions file for driving telescope jitter.

local tonumber = tonumber
local assert = assert

local pi = require('math').pi

local tables = require( 'saotrace.suplib.tables' )

local raygen = require( 'saotrace.raygen' )
local jitter = require( 'saotrace.raygen.jitter' );

local vobj = require( 'saotrace.raygen.validate').vobj:new()

local M = {}
setfenv( 1, M )

function dither_asol( ... )

   local ok, args = vobj:validate( { 
				      file = { type = 'string' },
				      ra   = { type = 'angle_ra' },
				      dec  = { type = 'angle_dec' },
				      roll = { type = 'angle_degree', default = 0 },
				   },
				   ... )
   assert( ok, args )

   raygen.jitter.file{ name = "asol",
		       file = args.file,

		       -- Tom Aldcroft says that we can extrapolate up
		       -- to 1/8 of a second outside of the dither
		       -- table.

		       -- But, it looks like Chandra asol files
		       -- contain dither bins with time stamp less
		       -- than TSTOP so the last time stamp may be
		       -- almost one bin width away from TSTOP.
		       extrapolate_max_dt = 0.256251,

		       quat_col = 'q_att',

		       optical_axis = {

			  {  angle = args.roll, axis = {  1,  0,  0 } },
			  {  angle = args.dec, axis = {  0, -1,  0 } },
			  {  angle = args.ra, axis = {  0,  0,  1 } },

		       },

		       coord_xfrm = {
			  -- transformations applied to input coords to get raytrace coords
			  x = { 0,  -1,  0 },
			  y = { 0,   0, -1 },
			  z = { -1,  0,  0 }
		       },

		    }
   raygen.jitter.roll( args.roll  )

end

function lissajous( args )

   raygen.jitter.lissajous( 'lj', args )

end


return { dither_asol = dither_asol,
	 lissajous = lissajous,
	 roll = raygen.jitter.roll
      }
